@@ -29,6 +29,7 @@ class SpatialJPEG(JPEG):
         # write content into temporary file
         tmp = tempfile.NamedTemporaryFile(suffix='jpeg')
         tmp.write(self.content)
+        tmp.flush()
         
         # colorspace
         if self.color_space is None:
@@ -40,7 +41,7 @@ class SpatialJPEG(JPEG):
             dither_mode = self.dither_mode.index
         # dct method
         dct_method = None
-        if dct_method is not None:
+        if self.dct_method is not None:
             dct_method = self.dct_method.index
             
         # allocate spatial
